package org.testng.xml.issue435;

import org.testng.annotations.Test;

public class Testclass {
    @Test(groups = "group1")
    public void testMethod1() {
    }

    @Test(groups = "group2")
    public void testMethod2() {
    }

    @Test(groups = "group3")
    public void testMethod3() {
    }

    @Test(groups = "group4")
    public void testMethod4() {
    }
}
