package test.reflect;

import org.testng.xml.XmlTest;

/**
 * Created on 12/30/15.
 *
 * @author <a href="mailto:nitin.matrix@gmail.com">Nitin Verma</a>
 */
public class XmlTestJustForTesting extends XmlTest {
}
