package org.testng;

/** @deprecated As of release 7.0.0, replaced by {@link org.testng.IConfigurationListener} */
@Deprecated
public interface IConfigurationListener2 extends IConfigurationListener {}
