package test.methodinterceptors.multipleinterceptors;

import org.testng.annotations.Test;

public class FooTest {

  @Test
  public void a() {
  }

  @Test
  public void b() {
  }

  @Test
  public void c() {
  }

  @Test
  public void d() {
  }
}
