package test.name;

import org.testng.Assert;
import org.testng.annotations.Test;

@Test(testName = "NAME")
public class NameSample {

  @Test
  public void test() {
    Assert.assertTrue(true);
  }
}
