package test.converter;

import junit.framework.TestCase;

public class ConverterSample1 extends TestCase {
  @Override
  protected void setUp() throws Exception {
    super.setUp();
  }

  @Override
  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public final void testClassJunit() {}

  public final void testSetClassId() {}

  public final void testSetClassName() {}
}
