package test.configuration.github1338;

import org.testng.annotations.Test;

public class ThirdGitHub1338Sample extends BaseGitHub1338Sample {

  @Test(groups = {"group1"})
  public void thirdTest() {}
}
